"""Tasks."""

from typing import List, Optional

from celery import shared_task
from moonmining.constants import EveTypeId
from moonmining.models.moons import Moon as MoonminingMoon

from eveuniverse.constants import EveGroupId
from eveuniverse.models import EveSolarSystem

from allianceauth.analytics.tasks import analytics_event
from allianceauth.services.hooks import get_extension_logger

from metenox.api.fuzzwork import BuySell, get_type_ids_prices
from metenox.esi import (
    DownTimeError,
    get_corporation_metenox_assets,
    get_metenox_from_esi,
    get_structure_info_from_esi,
)
from metenox.models import (
    EveTypePrice,
    HoldingCorporation,
    Metenox,
    MetenoxHourlyProducts,
    MetenoxStoredMoonMaterials,
    MetenoxTag,
    Moon,
)
from metenox.moons import get_metenox_hourly_harvest

logger = get_extension_logger(__name__)


class TaskError(Exception):
    """To be raised when a task fails"""


class FuelZeroedError(Exception):
    """
    Special exception to be raised when the fuel of a metenox should be 0ed out
    Made to check what the ESI asset input looked like
    """


@shared_task
def update_all_holdings():
    """
    Update all active owners on the application
    """
    holding_corps = HoldingCorporation.objects.filter(is_active=True)
    logger.info("Starting update for %s owner(s)", {len(holding_corps)})
    for holding in holding_corps:
        update_holding.delay(holding.corporation.corporation_id)


@shared_task
def update_holding(holding_corp_id: int):
    """
    Updated the list of metenoxes under a specific owner
    If harvest is set to True the harvest components are also recalculated
    """

    logger.info("Updating corporation id %s", holding_corp_id)

    holding_corp = HoldingCorporation.objects.get(
        corporation__corporation_id=holding_corp_id
    )

    if len(holding_corp.active_owners()) == 0:
        logger.info("No active owners for corporation id %s. Skipping", holding_corp_id)
        return

    try:
        metenoxes_info = get_metenox_from_esi(holding_corp)
    except DownTimeError:
        logger.warning("Currently at downtime. Exiting update")
        return

    metenoxes_info_dic = {
        metenox["structure_id"]: metenox for metenox in metenoxes_info
    }

    metenoxes_ids = set(metenox["structure_id"] for metenox in metenoxes_info)

    try:
        metenoxes_asset_dic = get_corporation_metenox_assets(
            holding_corp, metenoxes_ids
        )
    except DownTimeError:
        logger.warning("Currently at downtime. Exiting update")
        return

    current_metenoxes_ids = set(
        metenox.structure_id
        for metenox in Metenox.objects.filter(corporation=holding_corp)
    )

    disappeared_metenoxes_ids = (
        current_metenoxes_ids - metenoxes_ids
    )  # metenoxes that have been unanchored/destroyed/transferred
    Metenox.objects.filter(structure_id__in=disappeared_metenoxes_ids).delete()

    missing_metenoxes_ids = metenoxes_ids - current_metenoxes_ids
    for metenox_id in missing_metenoxes_ids:
        location_info = get_structure_info_from_esi(holding_corp, metenox_id)
        create_metenox.delay(
            holding_corp.corporation.corporation_id,
            metenoxes_info_dic[metenox_id],
            location_info,
        )

    metenoxes_to_updates = (
        current_metenoxes_ids - disappeared_metenoxes_ids - missing_metenoxes_ids
    )
    for metenox_id in metenoxes_to_updates:
        try:
            update_metenox.delay(
                metenox_id,
                metenoxes_info_dic[metenox_id],
                metenoxes_asset_dic[metenox_id],
            )
        except FuelZeroedError:
            logger.error(
                "Metenox id %d had the fuel set to 0. Full ESI asset: %s",
                metenox_id,
                metenoxes_asset_dic,
            )

    holding_corp.set_update_time_now()


@shared_task
def create_metenox(
    holding_corporation_id: int, structure_info: dict, location_info: dict
):
    """
    Creates and adds the Metenox in the database
    """
    holding_corporation = HoldingCorporation.objects.get(
        corporation__corporation_id=holding_corporation_id
    )
    logger.info(
        "Creating metenox %s for %s",
        structure_info["structure_id"],
        holding_corporation,
    )
    solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
        id=location_info["solar_system_id"]
    )
    try:
        nearest_celestial = solar_system.nearest_celestial(
            x=location_info["position"]["x"],
            y=location_info["position"]["y"],
            z=location_info["position"]["z"],
            group_id=EveGroupId.MOON,
        )
    except OSError as exc:
        logger.exception("%s: Failed to fetch nearest celestial", structure_info)
        raise exc

    if not nearest_celestial or nearest_celestial.eve_type.id != EveTypeId.MOON:
        logger.exception(
            "Couldn't find the moon corresponding to metenox %s", structure_info
        )
        raise TaskError(
            f"Couldn't fetch the metenox moon. Metenox id {structure_info['structure_id']}."
            f"Structure position {location_info['position']}"
        )

    eve_moon = nearest_celestial.eve_object
    moon, _ = Moon.objects.get_or_create(eve_moon=eve_moon)

    metenox = Metenox(
        moon=moon,
        structure_name=structure_info["name"],
        structure_id=structure_info["structure_id"],
        corporation=holding_corporation,
    )
    metenox.save()

    default_tags = MetenoxTag.objects.filter(default=True)

    metenox.tags.add(*default_tags)


@shared_task()
def update_metenox(
    metenox_structure_id: int,
    structure_info: dict,
    metenox_assets: Optional[List[dict]] = None,
):
    """
    Updates a metenox already existing in the database. Already receives the fetched ESI information of the structure
    """

    logger.info("Updating metenox id %s", metenox_structure_id)

    metenox = Metenox.objects.get(structure_id=metenox_structure_id)

    if metenox.structure_name != structure_info["name"]:
        logger.info("Updating metenox id %s name", metenox_structure_id)
        metenox.structure_name = structure_info["name"]

    # metenox.fuel_blocks_count = 0
    fuel_blocks = 0
    stored_products_to_update = []
    for asset in metenox_assets:
        if asset["location_flag"] == "StructureFuel":
            if asset["type_id"] == EveTypePrice.get_magmatic_gas_type_id():
                metenox.set_magmatic_gases(asset["quantity"])
            elif asset["type_id"] in EveTypePrice.get_fuel_blocs_type_ids():
                fuel_blocks += asset["quantity"]
        if asset["location_flag"] == "MoonMaterialBay":
            stored_moon_material, _ = MetenoxStoredMoonMaterials.objects.get_or_create(
                metenox=metenox, product_id=asset["type_id"]
            )
            stored_moon_material.amount = asset["quantity"]
            stored_products_to_update.append(stored_moon_material)

    if fuel_blocks == 0:
        logger.error(
            "Fuel at 0 detected in metenox id %d with assets %s",
            metenox_structure_id,
            metenox_assets,
        )
        raise FuelZeroedError

    metenox.set_fuel_blocs(fuel_blocks)

    MetenoxStoredMoonMaterials.objects.bulk_update(
        stored_products_to_update, fields=["amount"]
    )

    metenox.save()


@shared_task
def update_moon(moon_id: int, update_materials: bool = False):
    """
    Update the materials and price of a Moon
    If update_materials is set to true it will look in the moonmining app to update the composition
    """
    logger.info("Updating price of moon id %s", moon_id)

    moon = Moon.objects.get(eve_moon_id=moon_id)

    moon.update_price()

    # TODO write a test for this
    if update_materials:

        create_moon_materials(moon_id)


@shared_task
def create_moon_materials(moon_id: int):
    """
    Creates the materials of a moon without materials yet
    """

    moon = Moon.objects.get(eve_moon_id=moon_id)

    harvest = get_metenox_hourly_harvest(moon_id)

    # Delete all before as I have issues when using update_conflicts.
    # Backend doesn't seem compatible
    MetenoxHourlyProducts.objects.filter(moon=moon).delete()
    MetenoxHourlyProducts.objects.bulk_create(
        [
            MetenoxHourlyProducts(moon=moon, product=goo_type, amount=amount)
            for goo_type, amount in harvest.items()
        ]
    )

    moon.update_price()


@shared_task
def update_moons_from_moonmining():
    """
    Will fetch all the moons from aa-moonmining application and update the metenox database
    """

    logger.info("Updating all moons from moonmining")

    metenox_moons = Moon.objects.all()
    metenox_moon_ids = [moon.eve_moon.id for moon in metenox_moons]
    missing_moons = MoonminingMoon.objects.exclude(eve_moon__id__in=metenox_moon_ids)

    for moon in missing_moons:
        create_moon_from_moonmining.delay(moon.eve_moon.id)

    # metenox moons without a moonmining moon linked
    orphan_metenox_moons_ids = Moon.objects.filter(moonmining_moon=None).values_list(
        "eve_moon_id", flat=True
    )
    for moon in MoonminingMoon.objects.filter(eve_moon_id__in=orphan_metenox_moons_ids):
        orphan_metenox_moon = Moon.objects.get(eve_moon_id=moon.eve_moon_id)
        orphan_metenox_moon.moonmining_moon = moon
        orphan_metenox_moon.save()

    # creates data for moons that are missing their pulls if data is in the moonmining app
    moons_to_update = Moon.moons_in_need_of_update()
    for moon in moons_to_update:
        create_moon_materials.delay(moon.eve_moon.id)


@shared_task
def create_moon_from_moonmining(moon_id: int):
    """
    Fetches a moon from moonmining. Creates it for metenox and fetches materials
    """

    logger.info("Updating materials of moon id %s", moon_id)

    Moon.objects.get_or_create(
        eve_moon_id=moon_id,
        moonmining_moon=MoonminingMoon.objects.get(eve_moon_id=moon_id),
    )

    create_moon_materials(moon_id)


@shared_task
def update_prices():
    """Task fetching prices and then updating all moon values"""

    goo_ids = EveTypePrice.get_moon_goos_type_ids()

    goo_prices = get_type_ids_prices(goo_ids)

    for type_id, price in goo_prices.items():
        type_price, _ = EveTypePrice.objects.get_or_create(
            eve_type_id=type_id,
        )
        type_price.update_price(price)

    fuel_ids = EveTypePrice.get_fuels_type_ids()
    fuel_prices = get_type_ids_prices(fuel_ids, BuySell.SELL)

    for type_id, price in fuel_prices.items():
        type_price, _ = EveTypePrice.objects.get_or_create(
            eve_type_id=type_id,
        )
        type_price.update_price(price)

    moons = Moon.objects.all()
    logger.info(
        "Successfully updated goo and fuel prices. Now updating %s moons", moons.count()
    )

    for moon in moons:
        update_moon.delay(moon.eve_moon_id)


def send_analytics(label: str, value):
    """
    Send an analytics event
    """

    logger.info("Sending analytic %s with value %s", label, value)

    analytics_event(
        namespace="metenox.analytics",
        task="send_daily_stats",
        label=label,
        value=value,
        event_type="Stats",
    )


@shared_task
def send_daily_analytics():
    """
    Simple task starting the analytics work
    """

    logger.info("Starting the daily analytics task")

    count_moons = Moon.objects.count()
    count_holdings = HoldingCorporation.objects.count()
    count_metenoxes = Metenox.objects.count()

    send_analytics("moons", count_moons)
    send_analytics("holdings", count_holdings)
    send_analytics("metenoxes", count_metenoxes)



"""Corporations views"""

from django_datatables_view.base_datatable_view import BaseDatatableView

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.db import models
from django.db.models import Q
from django.http import JsonResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from allianceauth.eveonline.evelinks import dotlan
from allianceauth.services.hooks import get_extension_logger
from app_utils.views import link_html

from metenox.models import HoldingCorporation, Moon, Owner, Webhook
from metenox.views._helpers import corporation_breakdown_button_html
from metenox.views.general import add_common_context

from ._helpers import corporation_notifications_button_html, user_has_owner_in_corp

logger = get_extension_logger(__name__)


@login_required
@permission_required("metenox.view_metenoxes")
def list_corporations(request):
    """Will list all corporations and their statistics"""

    return render(request, "metenox/corporations.html", add_common_context())


# pylint: disable = too-many-ancestors
class CorporationListJson(
    PermissionRequiredMixin, LoginRequiredMixin, BaseDatatableView
):
    """Datatable view rendering corporations"""

    model = HoldingCorporation
    permission_required = "metenox.view_metenoxes"
    columns = [
        "id",
        "corporation_name",
        "alliance_name",
        "count_metenoxes",
        "raw_revenue",
        "profit",
        "breakdown",
        "notification",
    ]

    order_columns = [
        "pk",
        "",
        "",
        "",
        "",
        "",
        "",
    ]

    def render_column(self, row, column):
        if column == "id":
            return row.pk

        if column == "alliance_name":
            return self._render_alliance(row)

        if column == "corporation_name":
            return self._render_corporation(row)

        if column == "breakdown":
            return self._render_breakdown(row)

        if column == "notification":
            return self._render_notifications(row)

        return super().render_column(row, column)

    def _render_alliance(self, row: HoldingCorporation) -> str:
        alliance = row.corporation.alliance
        if alliance:
            alliance_link = link_html(
                dotlan.alliance_url(alliance.alliance_name), alliance.alliance_name
            )
            return alliance_link
        return ""

    def _render_corporation(self, row: HoldingCorporation) -> str:
        corporation_link = link_html(
            dotlan.corporation_url(row.corporation_name), row.corporation_name
        )
        return corporation_link

    def _render_breakdown(self, row: HoldingCorporation):
        if row.get_holding_tags():
            return corporation_breakdown_button_html(row)
        return None

    def _render_notifications(self, row):
        return corporation_notifications_button_html(row)

    def get_initial_queryset(self):
        return self.initial_queryset(self.request)

    @classmethod
    def initial_queryset(cls, request):
        """Basic queryset of the function"""
        holding_corporations_query = HoldingCorporation.objects.select_related(
            "corporation",
            "corporation__alliance",
        ).filter(is_active=True)

        if not request.user.has_perm("metenox.auditor"):
            user_owners = Owner.objects.filter(character_ownership__user=request.user)
            holding_corporations_query = holding_corporations_query.filter(
                owners__in=user_owners
            )

        return holding_corporations_query

    def filter_queryset(self, qs):
        """Use params in the GET to filter"""

        qs = self._apply_search_filter(qs, 0, "corporation__corporation_name")

        qs = self._apply_search_filter(qs, 1, "corporation__alliance__alliance_name")

        if search := self.request.GET.get("search[value]", None):
            qs = qs.filter(
                Q(corporation__corporation_name__icontains=search)
                | Q(corporation__alliance__alliance_name__icontains=search)
            )

        return qs

    def _apply_search_filter(self, qs, column_num, field) -> models.QuerySet:
        my_filter = self.request.GET.get(f"columns[{column_num}][search][value]", None)
        if my_filter:
            if self.request.GET.get(f"columns[{column_num}][search][regex]", False):
                kwargs = {f"{field}__iregex": my_filter}
            else:
                kwargs = {f"{field}__istartswith": my_filter}
            return qs.filter(**kwargs)
        return qs


@login_required
@permission_required("metenox.view_metenoxes")
def corporation_fdd_data(request) -> JsonResponse:
    """Provide lists for drop down fields"""
    qs = CorporationListJson.initial_queryset(request)
    columns = request.GET.get("columns")
    result = {}
    if columns:
        for column in columns.split(","):
            options = _calc_options(request, qs, column)
            result[column] = sorted(list(set(options)), key=str.casefold)
    return JsonResponse(result, safe=False)


# pylint: disable = too-many-return-statements, duplicate-code
def _calc_options(request, qs, column):
    if column == "alliance_name":
        values = qs.values_list("corporation__alliance__alliance_name", flat=True)
        values = (value for value in values if value)
        return values

    if column == "corporation_name":
        return qs.values_list("corporation__corporation_name", flat=True)

    return [f"** ERROR: Invalid column name '{column}' **"]


@login_required
@permission_required("metenox.corporation_manager")
def corporation_profit_breakdown(request, corporation_pk: int):
    """Render profit breakdown per tags for a corporation"""

    corporation = get_object_or_404(HoldingCorporation, pk=corporation_pk)
    tags = corporation.get_holding_tags()
    context = {
        "tags": {
            tag.name: {
                "count": corporation.count_metenoxes_with_tag(tag),
                "raw_profit": corporation.profit_metenoxes_with_tag(tag),
                "revenue": corporation.profit_metenoxes_with_tag(tag)
                - Moon.fuel_price() * corporation.count_metenoxes_with_tag(tag),
            }
            for tag in tags
        }
    }

    if request.GET.get("new_page"):
        context["title"] = "Corporation profit breakdown"
        context["content_file"] = "metenox/partials/corporation_breakdown.html"
        return render(
            request,
            "metenox/modals/generic_modal_page.html",
            add_common_context(context),
        )

    return render(
        request,
        "metenox/modals/corporation_breakdown.html",
        add_common_context(context),
    )


@login_required
@permission_required("metenox.view_metenoxes")
def corporation_notifications(request, corporation_pk: int):
    """Render notification details of a corporation"""

    corporation = get_object_or_404(HoldingCorporation, pk=corporation_pk)
    context = {"corporation": corporation}

    if request.GET.get("new_page"):
        context["title"] = "Corporation notifications"
        context["content_file"] = "metenox/partials/corporation_notifications.html"
        return render(
            request,
            "metenox/modals/generic_modal_page.html",
            add_common_context(context),
        )

    return render(
        request,
        "metenox/modals/corporation_notifications.html",
        add_common_context(context),
    )


@login_required
@permission_required("metenox.corporation_manager")
def change_corporation_ping_settings(request, corporation_pk: int):
    """Changes the ping settings of the corporation"""

    corporation = get_object_or_404(HoldingCorporation, pk=corporation_pk)

    if not user_has_owner_in_corp(request.user, corporation):
        messages.error(request, _("An error occurred"))
        return redirect("metenox:corporations")

    new_ping_on_remaining_magmatic_days = request.POST.get(
        "ping_on_remaining_magmatic_days"
    )
    new_ping_on_remaining_fuel_days = request.POST.get("ping_on_remaining_fuel_days")

    corporation.ping_on_remaining_magmatic_days = new_ping_on_remaining_magmatic_days
    corporation.ping_on_remaining_fuel_days = new_ping_on_remaining_fuel_days

    corporation.save()

    messages.success(
        request,
        _("Successfully changed %(corporation_name)s ping settings")
        % {"corporation_name": corporation.corporation_name},
    )

    return redirect(
        f"{reverse('metenox:notifications', args=[corporation_pk])}?new_page=true"
    )


@login_required
@permission_required("metenox.corporation_manager")
def add_webhook(request, corporation_pk: int):
    """Adds a new webhook to the corporation"""

    corporation = get_object_or_404(HoldingCorporation, pk=corporation_pk)
    corporation_name = corporation.corporation_name

    if not user_has_owner_in_corp(request.user, corporation):
        messages.error(request, _("An error occurred"))
        return redirect("metenox:corporations")

    if request.method == "POST":  # Will handle creating the webhook
        if creation_type := request.POST.get("type"):

            if creation_type == "new":
                webhook_url = request.POST.get("webhook_url")
                webhook_name = request.POST.get("webhook_name")
                Webhook.create_from_url(webhook_url, corporation, webhook_name)
                messages.success(
                    request,
                    _(
                        "Successfully created webhook %(webhook_name)s for corporation %(corporation_name)s"
                    )
                    % {
                        "webhook_name": webhook_name,
                        "corporation_name": corporation_name,
                    },
                )
                return redirect("metenox:add_corporation_webhook", corporation_pk)

            if creation_type == "existing":
                if webhook := Webhook.get_by_id(request.POST.get("webhook_id")):
                    webhook.add_corporation(corporation)
                    webhook_name = webhook.name
                    messages.success(
                        request,
                        _(
                            "Successfully added webhook %(webhook_name)s to corporation %(corporation_name)s"
                        )
                        % {
                            "webhook_name": webhook_name,
                            "corporation_name": corporation_name,
                        },
                    )
                    return redirect("metenox:add_corporation_webhook", corporation_pk)

                messages.error(request, _("Webhook id was not found"))
                return redirect("metenox:add_corporation_webhook", corporation_pk)

            logger.error(
                "Error when trying to create a webhook for corporation id %s."
                "The type parameter had an unexpected value %s",
                corporation_pk,
                creation_type,
            )
            messages.error(
                request,
                _(
                    "Unexpected argument received when creating a webhook for corporation %(corporation_name)s"
                )
                % {"corporation_name": corporation_name},
            )
            return redirect("metenox:corporations")

        logger.error(
            "Error when trying to add a webhook for corporation id %s. The type parameter was not received",
            corporation_pk,
        )
        messages.error(
            request,
            _("Error when trying to add a webhook to corporation %(corporation_name)s")
            % {"corporation_name": corporation_name},
        )
        return redirect("metenox:corporations")

    return render(
        request,
        "metenox/add_webhook.html",
        add_common_context(
            {
                "corporation": corporation,
            }
        ),
    )


@login_required
@permission_required("metenox.corporation_manager")
def remove_corporation_webhook(request, corporation_pk: int, webhook_pk: int):
    """Deletes the selected webhook from selected corporation"""

    corporation = get_object_or_404(HoldingCorporation, pk=corporation_pk)
    webhook = get_object_or_404(Webhook, pk=webhook_pk)

    if not user_has_owner_in_corp(request.user, corporation):
        messages.error(request, _("An error occurred"))
        return redirect("metenox:corporations")

    webhook.holding_corporations.remove(corporation)

    messages.success(
        request,
        _("Webhook %(webhook_name)s was successfully removed from %(corporation_name)s")
        % {
            "webhook_name": webhook.name,
            "corporation_name": corporation.corporation_name,
        },
    )

    return redirect(
        f"{reverse('metenox:notifications', args=[corporation_pk])}?new_page=true"
    )



"""Utility views or functions used everywhere"""

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import cache_page
from esi.decorators import token_required

from allianceauth.eveonline.models import EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from app_utils.allianceauth import notify_admins

from metenox import tasks
from metenox.app_settings import (
    METENOX_ADMIN_NOTIFICATIONS_ENABLED,
    METENOX_FUEL_BLOCKS_PER_HOUR,
    METENOX_MAGMATIC_GASES_PER_HOUR,
)
from metenox.models import ESI_SCOPES, HoldingCorporation, Moon, Owner, Webhook

logger = get_extension_logger(__name__)


@login_required
def index(request):
    """
    Basic view of the app.
    Should redirect depending on user's roles
    """

    if request.user.has_perm("metenox.view_moons"):
        return redirect("metenox:moons")

    if request.user.has_perm("metenox.view_metenoxes"):
        return redirect("metenox:metenoxes")

    logger.warning(
        "User %s tried to access the metenox application unauthorized", request.user
    )
    messages.error(request, _("An error occured"))
    return redirect("/")


def add_common_context(context: dict = None) -> dict:
    """Enhance the templates context with context that should be added to every page"""
    if context is None:
        context = {}

    if basic_title := context.get("page_title"):
        context["page_title"] = f"{basic_title} - Metenox"
    else:
        context["page_title"] = "Metenox"

    context["monthly_fuel_price"] = Moon.fuel_price()
    context["metenox_fuel_blocks_per_hour"] = METENOX_FUEL_BLOCKS_PER_HOUR
    context["metenox_magmatic_gases_per_hour"] = METENOX_MAGMATIC_GASES_PER_HOUR

    return context


@cache_page(3600)
def modal_loader_body(request):
    """Draw the loader body. Useful for showing a spinner while loading a modal."""
    return render(request, "metenox/modals/loader_body.html")


@permission_required(["metenox.view_metenoxes"])
@token_required(scopes=ESI_SCOPES)
@login_required
def add_owner(request, token):
    """Render view to add an owner."""
    character_ownership = get_object_or_404(
        request.user.character_ownerships.select_related("character"),
        character__character_id=token.character_id,
    )
    corporation_id = character_ownership.character.corporation_id
    try:
        corporation = EveCorporationInfo.objects.get(corporation_id=corporation_id)
    except EveCorporationInfo.DoesNotExist:
        corporation = EveCorporationInfo.objects.create_corporation(
            corp_id=corporation_id
        )
        corporation.save()

    holding_corporation, created_corporation = HoldingCorporation.objects.get_or_create(
        corporation=corporation,
    )

    if created_corporation:
        Webhook.add_default_webhooks_to_corporation(holding_corporation)

    owner, created_owner = Owner.objects.get_or_create(
        corporation=holding_corporation, character_ownership=character_ownership
    )
    if not created_owner:
        owner.enable()  # Gives another chance to the toon at being used for updates

    # TODO figure out why I need to type all this to get the right corp id
    tasks.update_holding.delay(owner.corporation.corporation.corporation_id)
    messages.success(request, f"Update of refineries started for {owner}.")
    if METENOX_ADMIN_NOTIFICATIONS_ENABLED:
        notify_admins(
            message=f"{owner} was added as new owner by {request.user}.",
            title=f"Metenox: Owner added: {owner}",
        )
    return redirect("metenox:corporations")



from django.contrib.auth.models import User
from django.test import TestCase

from allianceauth.authentication.models import CharacterOwnership
from app_utils.testing import create_fake_user

from metenox.models import Owner
from metenox.tests.utils import create_test_holding
from metenox.views._helpers import user_has_owner_in_corp


class TestCorporation(TestCase):

    def test_user_can_modify_corp(self):
        """
        Checks if the user_can_modify_corp is working as expected
        """

        holding_corporation = create_test_holding()

        user1: User = create_fake_user(character_id=10001, character_name="Test char 1")
        user2 = create_fake_user(character_id=10002, character_name="Test char 2")

        char1_ownership, _ = CharacterOwnership.objects.get_or_create(
            character=user1.profile.main_character, user=user1, owner_hash="fake_hash"
        )

        owner1 = Owner.objects.create(
            corporation=holding_corporation, character_ownership=char1_ownership
        )

        self.assertEqual(owner1.corporation, holding_corporation)

        self.assertTrue(user_has_owner_in_corp(user1, holding_corporation))
        self.assertFalse(user_has_owner_in_corp(user2, holding_corporation))

    def test_auditor_and_superuser_can_access_all(self):
        """
        Auditors and superusers should be able to access any metenoxes
        """

        holding_corporation = create_test_holding()

        auditor_user: User = create_fake_user(
            character_id=10001,
            character_name="Test char 1",
            permissions=["metenox.auditor"],
        )

        superuser = create_fake_user(character_id=10002, character_name="Test char 2")
        superuser.is_superuser = True
        superuser.save()

        self.assertTrue(auditor_user.has_perm("metenox.auditor"))
        self.assertTrue(user_has_owner_in_corp(auditor_user, holding_corporation))
        self.assertTrue(user_has_owner_in_corp(superuser, holding_corporation))



# flake8: noqa

from .corporations import (
    CorporationListJson,
    add_webhook,
    change_corporation_ping_settings,
    corporation_fdd_data,
    corporation_notifications,
    corporation_profit_breakdown,
    list_corporations,
    remove_corporation_webhook,
)
from .general import add_owner, index, modal_loader_body
from .metenoxes import MetenoxListJson, metenox_details, metenox_fdd_data, metenoxes
from .moons import MoonListJson, list_moons, moon_details, moons_fdd_data
from .prices import prices



from django.test import TestCase
from eveuniverse.models import EveType
from eveuniverse.tools.testdata import ModelSpec, create_testdata

from . import test_data_filename


class CreateEveUniverseTestData(TestCase):
    def test_create_testdata(self):
        moon_goo_id_range = [i for i in range(16633, 16654)]
        moon_goo_id_range.remove(16645)  # why does this one not exists? #CCPLZ
        testdata_spec = [
            ModelSpec("EveMoon", ids=[40178441]),  # Loads Tama
            ModelSpec("EveType", ids=moon_goo_id_range),  # all moon goo
            ModelSpec("EveType", ids=[81143]),  # Magmatic gases
            ModelSpec(
                "EveType",
                ids=[
                    45490,
                    45491,
                    45492,
                    45493,
                    46280,
                    46281,
                    46282,
                    46283,
                    46284,
                    46285,
                    46286,
                    46287,
                    62454,
                    62455,
                    62456,
                    62457,
                    62458,
                    62459,
                    62460,
                    62461,
                    62463,
                    62464,
                    62466,
                    62467,
                ],
                enabled_sections=[EveType.Section.TYPE_MATERIALS],
            ),  # Ubiquitous Moon Asteroids
            ModelSpec(
                "EveType",
                ids=[
                    45494,
                    45495,
                    45496,
                    45497,
                    46288,
                    46289,
                    46290,
                    46291,
                    46292,
                    46293,
                    46294,
                    46295,
                    62468,
                    62469,
                    62470,
                    62471,
                    62472,
                    62473,
                    62474,
                    62475,
                    62476,
                    62477,
                    62478,
                    62479,
                ],
                enabled_sections=[EveType.Section.TYPE_MATERIALS],
            ),  # Common Moon Asteroids.Section.TYPE_MATERIALS
            ModelSpec(
                "EveType",
                ids=[
                    45498,
                    45499,
                    45500,
                    45501,
                    46296,
                    46297,
                    46298,
                    46299,
                    46300,
                    46301,
                    46302,
                    46303,
                    62480,
                    62481,
                    62482,
                    62483,
                    62484,
                    62485,
                    62486,
                    62487,
                    62488,
                    62489,
                    62490,
                    62491,
                ],
                enabled_sections=[EveType.Section.TYPE_MATERIALS],
            ),  # Uncommon Moon Asteroids.Section.TYPE_MATERIALS
            ModelSpec(
                "EveType",
                ids=[
                    45510,
                    45511,
                    45512,
                    45513,
                    46312,
                    46313,
                    46314,
                    46315,
                    46316,
                    46317,
                    46318,
                    46319,
                    62504,
                    62505,
                    62506,
                    62507,
                    62508,
                    62509,
                    62510,
                    62511,
                    62512,
                    62513,
                    62514,
                    62515,
                ],
                enabled_sections=[EveType.Section.TYPE_MATERIALS],
            ),  # Exceptional Moon Asteroids
        ]
        create_testdata(testdata_spec, test_data_filename())



from django.test import TestCase

from metenox.models import MetenoxTag
from metenox.tests.testdata.load_eveuniverse import load_eveuniverse

from .utils import create_test_metenox


class TestTags(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_eveuniverse()

    def test_get_holding_tags(self):
        tag1 = MetenoxTag.objects.create(name="Tag1")
        tag2 = MetenoxTag.objects.create(name="Tag2")

        metenox = create_test_metenox()
        holding = metenox.corporation

        self.assertEqual([], holding.get_holding_tags())

        metenox.tags.add(tag1)

        tag_list = holding.get_holding_tags()

        self.assertIn(tag1, tag_list)
        self.assertNotIn(tag2, tag_list)

    def test_count_metenox_tags(self):
        tag1 = MetenoxTag.objects.create(name="Tag1")
        tag2 = MetenoxTag.objects.create(name="Tag2")

        metenox = create_test_metenox()
        holding = metenox.corporation

        self.assertEqual(0, holding.count_metenoxes_with_tag(tag1))
        self.assertEqual(0, holding.count_metenoxes_with_tag(tag2))

        metenox.tags.add(tag1)

        self.assertEqual(1, holding.count_metenoxes_with_tag(tag1))
        self.assertEqual(0, holding.count_metenoxes_with_tag(tag2))



from django.core.management.base import BaseCommand

from allianceauth.services.hooks import get_extension_logger

from metenox import tasks

logger = get_extension_logger(__name__)


class Command(BaseCommand):
    help = "Fetches new moon goo prices and update metenox harvest values"

    def handle(self, *args, **options):
        logger.info("Command to update prices received")
        tasks.update_prices.delay()
